#include "stdio.h"
#include "string.h"
#include "assert.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "samples.h"
#include "animscript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


int current_ID=0;


PLACED_OBJECT::PLACED_OBJECT(void)
{
	ID=current_ID++;

	x=0;
	y=0;
	object=0;
	room=0;
	interf=0;
	game=0;
} /* PLACED_OBJECT::PLACED_OBJECT */ 


PLACED_OBJECT::PLACED_OBJECT(long int nx,long int ny,OBJECT *o,MOG2 *g,ROOM *r,MOG2_INTERFACE *i)
{
	ID=current_ID++;

	x=nx;
	y=ny;
	object=o;
	game=g;
	room=r;
	interf=i;
} /* PLACED_OBJECT::PLACED_OBJECT */ 


PLACED_OBJECT::PLACED_OBJECT(PLACED_OBJECT &o)
{
	ID=current_ID++;

	x=o.x;
	y=o.y;
	object=o.object;
	game=o.game;
	room=o.room;
	interf=o.interf;
} /* PLACED_OBJECT::PLACED_OBJECT */ 


PLACED_OBJECT::~PLACED_OBJECT(void)
{
} /* PLACED_OBJECT::~PLACED_OBJECT */ 


bool PLACED_OBJECT::save(FILE *fp,MOG2 *game)
{
	int n;
	List<VARIABLE> l;
	VARIABLE *v;

	n=game->objects.SearchObjRef(object);
	assert(n>=0);
	fprintf(fp,"%i %i %i\n",int(x),int(y),n);
	fprintf(fp,"%i\n",modified_variables.Length());

	l.Instance(modified_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		v->save(fp);
	} /* while */ 

	return true;
} /* PLACED_OBJECT::save */ 


bool PLACED_OBJECT::load(FILE *fp,MOG2 *g,ROOM *r,MOG2_INTERFACE *i_f)
{
	int i,xtmp,ytmp;
	int tmp;
	VARIABLE *v;

	if (3!=fscanf(fp,"%i %i %i",&xtmp,&ytmp,&tmp)) return false;
	x=xtmp;
	y=ytmp;

	object=g->objects[tmp];
	game=g;
	room=r;
	interf=i_f;

	if (1!=fscanf(fp,"%i",&tmp)) return false;

	for(i=0;i<tmp;i++) {
		v=new VARIABLE();
		v->load(fp);
		modified_variables.Add(v);
	} /* for */ 

	return true;
} /* PLACED_OBJECT::load */ 

 
void PLACED_OBJECT::draw(SDL_Surface *sfc)
{
	if (object!=0) {
		if (object->control==CTRL_TEXT ||
			object->control==CTRL_GAMEWINDOW) {
		} else {
			int i,j;
			object->get_link(object->default_tile,0,0,&i,&j);
			object->draw(x+i,y+j,object->default_tile,0,sfc,0);
		} /* if */ 
	} /* if */ 
} /* PLACED_OBJECT::draw */ 


void PLACED_OBJECT::draw(long int sx,long int sy,SDL_Surface *sfc)
{
	if (object!=0) {
		if (object->control==CTRL_TEXT) {
			text_draw(sx,sy,sfc,0,game);
		} else {
			if (object->control==CTRL_GAMEWINDOW) {
				object->gw_draw(sx+x,sy+y,sfc,0);
			} else {
				if (object->control==CTRL_MENU) {
					object->menu_draw(sx+x,sy+y,sfc,0);
				} else {
					int i,j;
					object->get_link(object->default_tile,0,0,&i,&j);
					object->draw(sx+x+i,sy+y+j,object->default_tile,0,sfc,0);
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* if */ 
} /* PLACED_OBJECT::draw */ 


void PLACED_OBJECT::draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc)
{
	if (object!=0) {
		int i,j;
		object->get_link(object->default_tile,0,0,&i,&j);
		object->draw_collision_mask(sx+x+i,sy+y+j,object->default_tile,sfc,0);
	} /* if */ 
} /* PLACED_OBJECT::draw_collision_mask */ 


void PLACED_OBJECT::draw_scaled(long int sx,long int sy,SDL_Surface *dest,float scale)
{
	if (object!=0) {
		int i,j;
		object->get_link(object->default_tile,0,0,&i,&j);
		object->draw_transform(int(sx+(x+i)*scale),int(sy+(y+j)*scale),object->default_tile,0,dest,0,scale,0);
	} /* if */ 
} /* PLACED_OBJECT::draw_Scaled */ 


int PLACED_OBJECT::get_dx(void)
{
	return object->get_dx(object->default_tile);
} /* PLACED_OBJECT::get_dx */ 


int PLACED_OBJECT::get_dy(void)
{
	return object->get_dy(object->default_tile);
} /* PLACED_OBJECT::get_dy */ 


void PLACED_OBJECT::get_gw_bbox(int *x1,int *y1,int *x2,int *y2)
{
	*x1=x;
	*y1=y;
	*x2=x+object->gw_dx;
	*y2=y+object->gw_dy;
} /* PLACED_OBJECT::get_gw_bbox */ 


void PLACED_OBJECT::get_menu_bbox(int *x1,int *y1,int *x2,int *y2)
{
	*x1=x;
	*y1=y;
	*x2=x+object->menu_dx;
	*y2=y+object->menu_dy;
} /* PLACED_OBJECT::get_menu_bbox */ 
