#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"

#include "GVariableWnd.h"
#include "GMapWnd.h"
#include "GTopWnd.h"

#include "MTopWnd.h"
#include "MVariableWnd.h"
#include "MRoomsWnd.h"

#include "OTopWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

const int mini_map_size=128;

MRoomsWnd::MRoomsWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;

	editor=e;

	b=new BButton(160,0,90,12,"NEW ROOM",1);
	buttons.Add(b);

} /* MRoomsWnd::MRoomsWnd */ 


MRoomsWnd::~MRoomsWnd()
{
} /* MRoomsWnd::~MRoomsWnd */ 


bool MRoomsWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (mouse_button&SDL_BUTTON(3) &&
		!(old_mouse_button&SDL_BUTTON(3)) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<ROOM> l,todelete;
		ROOM *r;
		int x=4;
	
		l.Instance(editor->map->rooms);
		l.Rewind();
		while(l.Iterate(r)) {
			if (r->x==NOPOSITION || r->y==NOPOSITION) {
				if (local_mouse_y>=TTF_FontHeight(font)+6 && 
					local_mouse_y<=TTF_FontHeight(font)+6+r->dy*2+4 &&
					local_mouse_x>=x &&
					local_mouse_x<=int(x+r->dx*2+4)) {
					BVChoiceDlg *dlg;
					int res=0;

					dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Place in map\nEdit room\nCopy\nDelete\n",&res,font);
					dlg->Execute();
					delete dlg;

					editor->mv_selected=0;
					set_dirty();

					if (res==1) {
						editor->mv_selected=r;
					} /* if */ 

					if (res==2) {
						editor->room=r;
						editor->view=ROOM_VIEW;
						editor->rv_selected_obj=0;
						editor->rv_selected_tile=0;
						editor->set_all_dirty();
					} /* if */ 
					
					if (res==3) {
						ROOM *nr;

						nr=new ROOM(*r);
						editor->map->rooms.Add(nr);
					} /* if */ 

					if (res==4) {
						todelete.Add(r);
					} /* if */ 
				} /* if */ 

				x+=r->dx*2+8;
			} /* if */ 
		} /* while */ 

		while(!todelete.EmptyP()) {
			ROOM *r;
			r=todelete.ExtractIni();
			editor->map->rooms.DeleteElement(r);
			delete r;
		} /* while */ 

	} /* if */ 

	return true;
} /* GMapWnd::cycle */ 


void MRoomsWnd::contens_size(int *dx,int *dy)
{
	int x;
	ROOM *r;
	List<ROOM> l;
	int max_x=0;
	int max_y=0;
	
	BWindow::contens_size(dx,dy);
	
	l.Instance(editor->map->rooms);
	l.Rewind();
	x=4;
	while(l.Iterate(r)) {
		if (r->x==NOPOSITION || r->y==NOPOSITION) {
			if ((TTF_FontHeight(font)+6+r->dy*2+4+2)>max_y) max_y=(TTF_FontHeight(font)+6+r->dy*2+4+2);
			x+=r->dx*2+8;
		} /* if */ 
	} /* while */ 

	if (x>max_x) max_x=x;

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* MRoomsWnd::contens_size */ 


void MRoomsWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int x;
	ROOM *r;
	List<ROOM> l;
	Uint32 color;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	l.Instance(editor->map->rooms);
	l.Rewind();
	x=4;
	while(l.Iterate(r)) {
		if (r->x==NOPOSITION || r->y==NOPOSITION) {
			/* Draw room miniature: */ 
			if (editor->show_miniatures) r->draw(sfc,x+2-stx,TTF_FontHeight(font)+8-sty,r->dx*2,r->dy*2,0,0,0.125f,0);
									else r->draw_frame(sfc,x+2-stx,TTF_FontHeight(font)+8-sty,r->dx*2,r->dy*2,0,0,0.125f);
			if (r==editor->mv_selected) color=0x00ff0000;
						           else color=0x00ffffff;
			draw_rectangle(sfc,x-stx,TTF_FontHeight(font)+6-sty,r->dx*2+3,r->dy*2+3,color);
			x+=r->dx*2+8;
		} /* if */ 
	} /* while */ 

	print_left("UNPLACED ROOMS:",font,255,255,255,sfc,3-stx,0-sty);
} /* MRoomsWnd::draw_contens */ 


bool MRoomsWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* New Room!!: */ 
		{
			ROOM *r;

			r=new ROOM(editor->map,32,20);
			editor->map->rooms.Add(r);
			editor->mv_selected=0;
			set_dirty();
		}
		break;

	default:
		break;
	} /* switch */ 

	return true;
} /* MRoomsWnd::button_pressed */ 
